## This package handles monika.conf file.
## it uses ConfNode.pm to store nodes description got from the configuration file.
package Conf;

use strict;
use warnings;
use AppConfig qw(:expand :argcount);
use monika::ConfNode;

## class constructor
my $myself;

sub new {
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $self = {};
  $self->{ALLNODES} = {};
  bless ($self, $class);
  $myself = $self;
  return $self;
}

sub myself () {
  defined $myself or die "do first a new and parse please...";
  return $myself;
}

## parse config file
sub parse {
  my $self = shift;
  if (@_) {
    $self->{FILE} = shift;
  } else {
    $self->{FILE} = "/etc/monika.conf";
  }
  my $config = AppConfig->new({
				  GLOBAL => {
					     DEFAULT  => "<unset>",
					     ARGCOUNT => ARGCOUNT_ONE,
					    }
				 });
  $config->define("clustername",
		  {
		   DEFAULT => "Cluster"
		  });
  $config->define("batchscheduler",
		  {
		   DEFAULT => "PBS"
		  });
  $config->define("nodes_per_line",
		  {
		   DEFAULT => 10
		  });
  $config->define("nodename_regex",
		  {
		   DEFAULT => '(\d+)'
		  });
  $config->define("nodename_regex_display",
		  {
		   DEFAULT => '(.+)'
		  });
  $config->define("pbsnodes",
		  {
		   DEFAULT => "pbsnodes -a"
		  });
  $config->define("qstat",
		  {
		   DEFAULT => "qstat -f"
		  });
  $config->define("server_do_mail",
		  {
		   DEFAULT => "no",
		  });
  $config->define("node_group",
		  {
		   ARGCOUNT => ARGCOUNT_HASH
		  });
  $config->define("default_state",
		  {
		   ARGCOUNT => ARGCOUNT_HASH
		  });
  $config->define("set_color",
		  {
		   ARGCOUNT => ARGCOUNT_HASH
		  });
  $config->define("color_pool",
		  {
		   ARGCOUNT => ARGCOUNT_LIST
		  });
  $config->define("hidden_property",
		  {
		   ARGCOUNT => ARGCOUNT_LIST
		  });
  $config->file($self->{FILE});

  $self->{CLUSTERNAME} = $config->clustername();
  $self->{BATCHSCHEDULER} = $config->batchscheduler();
  $self->{NODESPERLINE} = $config->nodes_per_line();
  my $regex = $config->nodename_regex();
  $self->{NODENAMEREGEX} = qr/$regex/;
  my $regex_display = $config->nodename_regex_display();
  $self->{NODENAMEREGEXDISPLAY} = qr/$regex_display/;
  $self->{PBSNODESCMD} = $config->pbsnodes();
  $self->{QSTATCMD} = $config->qstat();
  $self->{SERVER_DO_MAIL} = $config->server_do_mail();
  $self->{NODE_GROUP} = $config->node_group();
  $self->{DEFAULT_STATE} = $config->default_state();
  $self->{SET_COLOR} = $config->set_color();
  $self->{COLOR_POOL} = $config->color_pool();
  $self->{HIDDEN_PROPERTIES} = $config->hidden_property();

  my $allnodes = $self->{ALLNODES};

  foreach my $nodeType (keys %{$self->{NODE_GROUP}}) {
    my $state;
    if ($self->{DEFAULT_STATE}->{$nodeType}) {
      $state = $self->{DEFAULT_STATE}->{$nodeType};
    } else {
      $state = $nodeType;
    }
    my @nodes = split /\s+/,$self->{NODE_GROUP}->{$nodeType};
    foreach (@nodes) {
      if (/^(\d+)-(\d+)$/) {
	foreach ($1..$2) {
	  $allnodes->{$_} = ConfNode->new($_,$state);
	}
      } else {
	$allnodes->{$_} = ConfNode->new($_,$state);
      }
    }
  }
  return 1;
}

## return cluster name
sub clustername {
  my $self = shift;
  return $self->{CLUSTERNAME};
}

## return the batscheduler name
sub batchscheduler {
  my $self = shift;
  return $self->{BATCHSCHEDULER};
}

## return the number of node to be diplayed per line in the reservation table
sub nodes_per_line {
  my $self = shift;
  return $self->{NODESPERLINE};
}

## return the regex to extract a pbs node display name from its real name
sub nodenameRegex {
  my $self = shift;
  return $self->{NODENAMEREGEX};
}

## return the regex to extract a the name displayed on the www page
sub nodenameRegexDisplay {
  my $self = shift;
  return $self->{NODENAMEREGEXDISPLAY};
}

## return pbsnodes command line as set in config file
sub pbsnodesCmd {
  my $self = shift;
  return $self->{PBSNODESCMD};
}

## return qstat command line as set in config file
sub qstatCmd {
  my $self = shift;
  return $self->{QSTATCMD};
}

## return true if config file says pbs server also is a mail server
sub server_do_mail {
  my $self = shift;
  $_ = $self->{SERVER_DO_MAIL};
  return (/^\s*yes\s*$/i or /^\s*true\s*$/i);
}

## return a hash containing (state,color) couples
sub colorHash {
  my $self = shift;
  return $self->{SET_COLOR};
}

sub colorPool {
  my $self = shift;
  return $self->{COLOR_POOL};
}

## return the list of properties not to be shown in the property chooser
sub hiddenProperties {
	my $self = shift;
	return @{$self->{HIDDEN_PROPERTIES}};
}

## return a hash containing all node descriptions got from the config file
sub allnodes {
  my $self = shift;
  return $self->{ALLNODES};
}

## that's all.
return 1;
